# -*- coding:utf-8 -*-
import yaml

from value import *
from restsql.config.load import *
from manager import update_schema

logger = logging.getLogger('config')
logger.setLevel(LOG_LEVEL)

def load_config(path):
    """
    载入配置文件。本模块唯一对外暴露函数。
    :param path: 配置文件目录
    :return: None
    """
    with open(path, "r") as fp:  # 配置文件由configmap挂载到镜像路径后传入
        config = yaml.safe_load(fp)
        logger.debug("载入配置文件: %s", config)
    need_manage = {}
    # 载入数据源
    databases = config.get('databases', [])
    for database in databases:
        name = database.get('name')
        db_type = database.get('db_type')
        host = database.get('host', '127.0.0.1')
        port = database.get('port', 1024)
        user = database.get('user', '')
        password = database.get('password', '')
        db_name = database.get('db_name', '')
        auto_manage = database.get('auto_manage', False)
        black_tables = database.get('black_tables', [])
        black_fields = database.get('black_fields', {})
        register_database(name=name, db_type=db_type, host=host, port=port, user=user, password=password, db_name=db_name)
        logger.info('载入数据源%s', name)
        if auto_manage:
            need_manage[name] = (db_type, black_tables, black_fields)
    # 载入数据表
    tables = config.get('tables', [])
    for table in tables:
        name = table.get('name')
        table_name = table.get('table_name')
        belong_to = table.get('belong_to')
        fields = table.get('fields')
        register_table(name=name, table_name=table_name, fields=fields, belong_to=belong_to)
        logger.info('载入数据表%s', name)
    # 自动载入数据表
    for (name, tup_black) in need_manage.items():
        logger.info('自动维护数据源')
        update_schema(name, *tup_black)
