# -*- coding:utf-8 -*-
from restsql import ProgramConfigException
from value import *
from restsql.config.load import *
from restsql.config.value import *

logger = logging.getLogger('manager')
logger.setLevel(LOG_LEVEL)

_KNOWN_FIELD_TYPE = {
    'int': 'int',
    'tinyint': 'int',
    'bigint': 'int',
    'double': 'float',
    'string': 'string',
    'bool': 'bool',
    'timestamp': 'timedelta',
    'datetime': 'datetime',
}


def update_schema(name, db_type, black_tables, black_fields):
    """
    自动添加schema。根据传入数据源需求，自动数据源的schema，并且忽视黑名单中的表，与黑名单中的字段。
    目前仅仅支持impala。
    :return:
    """
    logger.info('自动添加schema激活, 维护数据源%s中', name)
    database = index4database.get(name)
    if DatabaseType.MYSQL.value == db_type:
        pass
    elif DatabaseType.POSTGRESQL.value == db_type:
        pass
    elif DatabaseType.ELASTICSEARCH.value == db_type:
        pass
    elif DatabaseType.IMPALA.value == db_type:
        conn = database.conn
        cur = conn.cursor()
        cur.execute('show tables')  # 发送请求获取所有table
        all_tables = cur.fetchall()
        for table_tuple in all_tables:
            table_name = table_tuple[0]
            fields = {}
            # 对所有不在black_tables里、tables里的表发送查询schema请求
            if '{}.{}'.format(name, table_name) not in index4table and table_name not in black_tables:
                cur.execute('describe {}'.format(table_name))
                records = cur.fetchall()
                for record in records:  # 解析结果，并根据black_fields添加schema
                    if record[0] not in black_fields and \
                            record[1] in _KNOWN_FIELD_TYPE:  # TODO: record type -> 1?
                        fields[record[0]] = _KNOWN_FIELD_TYPE[record[1]]
            register_table(name=table_name, table_name=table_name, fields=fields, belong_to=name)
        cur.close()
    else:
        raise ProgramConfigException(1, '未知数据源类型')

